@@ -3,13 +3,11 @@ from __future__ import unicode_literals
 import six
 from base64 import b64encode
 from django.forms import forms
-from django.forms import fields
-from django.forms import widgets
 from django.http import QueryDict
+from django.utils.importlib import import_module
 from django.utils.html import format_html
 from django.utils.encoding import python_2_unicode_compatible, force_text
-from django.utils.safestring import mark_safe, SafeData, SafeText
-from djangular.forms.widgets import CheckboxSelectMultiple as DjngCheckboxSelectMultiple
+from django.utils.safestring import mark_safe, SafeData
 
 
 class SafeTuple(SafeData, tuple):
@@ -22,14 +20,14 @@ class SafeTuple(SafeData, tuple):
 @python_2_unicode_compatible
 class TupleErrorList(list):
     """
-    A list of errors, which in comparison to Django's ErrorList contains a tuple for each item.
+    A list of errors, which in contrast to Django's ErrorList, contains a tuple for each item.
     This tuple consists of the following fields:
     0: identifier: This is the model name of the field.
     1: The CSS class added to the embedding <ul>-element.
-    2: property: $pristine or $dirty used by ng-show on the wrapping <ul>-element.
+    2: property: '$pristine', '$dirty' or None used by ng-show on the wrapping <ul>-element.
     3: An arbitrary property used by ng-show on the actual <li>-element.
     4: The CSS class added to the <li>-element.
-    5: The used error message. If this contains the magic word '$message' it will be added with
+    5: The desired error message. If this contains the magic word '$message' it will be added with
        ``ng-bind`` rather than rendered inside the list item.
     """
     ul_format = '<ul class="{1}" ng-show="{0}.{2}" ng-cloak>{3}</ul>'
@@ -45,22 +43,15 @@ class TupleErrorList(list):
     def as_ul(self):
         if not self:
             return ''
-        pristine_list_items = []
-        dirty_list_items = []
+        error_lists = {'$pristine': [], '$dirty': []}
         for e in self:
             li_format = e[5] == '$message' and self.li_format_bind or self.li_format
             err_tuple = (e[0], e[3], e[4], force_text(e[5]))
-            if e[2] == '$pristine':
-                pristine_list_items.append(format_html(li_format, *err_tuple))
-            else:
-                dirty_list_items.append(format_html(li_format, *err_tuple))
+            error_lists[e[2]].append(format_html(li_format, *err_tuple))
         # renders and combine both of these lists
         first = self[0]
-        return (pristine_list_items and
-          format_html(self.ul_format, first[0], first[1], '$pristine', mark_safe(''.join(pristine_list_items)))
-            or SafeText()) + (dirty_list_items and
-          format_html(self.ul_format, first[0], first[1], '$dirty', mark_safe(''.join(dirty_list_items)))
-            or SafeText())
+        return mark_safe(''.join([format_html(self.ul_format, first[0], first[1], prop,
+                    mark_safe(''.join(list_items))) for prop, list_items in error_lists.items()]))
 
     def as_text(self):
         if not self:
@@ -101,10 +92,6 @@ class NgBoundField(forms.BoundField):
             css_classes = getattr(self.form, 'widget_css_classes', None)
         if css_classes:
             attrs.update({'class': css_classes})
-        # transfer error state from bound field to AngularJS validation
-        errors = [e for e in self.errors if e[3] == '$pristine']
-        if errors:
-            attrs.update({'djng-error': errors[0][4]})
         return super(NgBoundField, self).as_widget(widget, attrs, **kwargs)
 
     def label_tag(self, contents=None, attrs=None, label_suffix=None):
@@ -118,6 +105,24 @@ class NgBoundField(forms.BoundField):
 class NgFormBaseMixin(object):
     form_error_css_classes = 'djng-form-errors'
     field_error_css_classes = 'djng-field-errors'
+    field_mixins_module = field_mixins_fallback_module = 'djangular.forms.field_mixins'
+
+    def __new__(cls, **kwargs):
+        field_mixins_module = import_module(cls.field_mixins_module)
+        field_mixins_fallback_module = import_module(cls.field_mixins_fallback_module)
+        new_cls = super(NgFormBaseMixin, cls).__new__(cls)
+        # add additional methods to django.form.fields at runtime
+        for field in new_cls.base_fields.values():
+            FieldMixinName = field.__class__.__name__ + 'Mixin'
+            try:
+                FieldMixin = getattr(field_mixins_module, FieldMixinName)
+            except AttributeError:
+                try:
+                    FieldMixin = getattr(field_mixins_fallback_module, FieldMixinName)
+                except AttributeError:
+                    FieldMixin = field_mixins_fallback_module.DefaultFieldMixin
+            field.__class__ = type(field.__class__.__name__, (field.__class__, FieldMixin), {})
+        return new_cls
 
     def __init__(self, data=None, *args, **kwargs):
         try:
@@ -128,7 +133,11 @@ class NgFormBaseMixin(object):
         self.form_name = kwargs.pop('form_name', form_name)
         error_class = kwargs.pop('error_class', TupleErrorList)
         kwargs.setdefault('error_class', error_class)
-        data = self.convert_widgets(data)
+        self.convert_widgets()
+        if isinstance(data, QueryDict):
+            data = self.rectify_multipart_form_data(data.copy())
+        elif isinstance(data, dict):
+            data = self.rectify_ajax_form_data(data.copy())
         super(NgFormBaseMixin, self).__init__(data, *args, **kwargs)
 
     def __getitem__(self, name):
@@ -162,20 +171,45 @@ class NgFormBaseMixin(object):
 
     def get_widget_attrs(self, bound_field):
         """
-        Return a dictionary of additional widget attributes.
+        Return a dictionary of additional attributes which shall be added to the widget,
+        used to render this field.
         """
         return {}
 
-    def convert_widgets(self, data):
+    def convert_widgets(self):
         """
         During form initialization, some widgets have to be replaced by a counterpart suitable to
         be rendered the AngularJS way.
         """
+        for field in self.base_fields.values():
+            try:
+                new_widget = field.get_converted_widget()
+            except AttributeError:
+                pass
+            else:
+                if new_widget:
+                    field.widget = new_widget
+
+    def rectify_multipart_form_data(self, data):
+        """
+        If a widget was converted and the Form data was submitted through a multipart request,
+        then these data fields must be converted to suit the Django Form validation
+        """
         for name, field in self.base_fields.items():
-            if isinstance(field, fields.MultipleChoiceField) and isinstance(field.widget, widgets.CheckboxSelectMultiple):
-                fw_dict = field.widget.__dict__
-                field.widget = DjngCheckboxSelectMultiple()
-                field.widget.__dict__ = fw_dict
-                if isinstance(data, QueryDict):
-                    data = field.widget.implode_multi_values(name, data.copy())
+            try:
+                field.widget.implode_multi_values(name, data)
+            except AttributeError:
+                pass
+        return data
+
+    def rectify_ajax_form_data(self, data):
+        """
+        If a widget was converted and the Form data was submitted through an Ajax request,
+        then these data fields must be converted to suit the Django Form validation
+        """
+        for name, field in self.base_fields.items():
+            try:
+                data[name] = field.widget.convert_ajax_data(data.get(name, {}))
+            except AttributeError:
+                pass
         return data
