@@ -14,6 +14,7 @@ class NgModelFormMixin(NgFormBaseMixin):
     If form validation fails, the ErrorDict is rewritten in a way, so that the Angular controller
     can access the error strings using the same key values as for its models.
     """
+    add_djng_error = False
 
     def __init__(self, data=None, *args, **kwargs):
         self.scope_prefix = kwargs.pop('scope_prefix', getattr(self, 'scope_prefix', None))
@@ -69,13 +70,13 @@ class NgModelFormMixin(NgFormBaseMixin):
         return errors
 
     def get_widget_attrs(self, bound_field):
+        attrs = super(NgModelFormMixin, self).get_widget_attrs(bound_field)
         identifier = self.add_prefix(bound_field.name)
         ng = {
             'name': bound_field.name,
             'identifier': identifier,
             'model': self.scope_prefix and ('%s.%s' % (self.scope_prefix, identifier)) or identifier
         }
-        attrs = {}
         if hasattr(self, 'Meta') and bound_field.name in getattr(self.Meta, 'ng_models', []):
             attrs['ng-model'] = ng['model']
         for key, fmtstr in self.ng_directives.items():
